/** ********************************************************************************************************************
 * @file        MQTTSimplified.ino
 * @author      Briowireless, Inc.
 * @date        2017-01-09
 * @brief       Main program file for MQTT Sample.
 * @version     API V1
 * @copyright   Briowireless, Inc., 2017, Licensed under BSD license. See LICENSE file for more details.
 *
 ** *******************************************************************************************************************/


// Include directives  -------------------------------------------------------------------------------------------------
#include "MQTTClient.h"

#include <SoftwareSerial.h>

// Preprocessor definitions --------------------------------------------------------------------------------------------

#define LED_PIN                             13
#define SOFT_RXPIN 11
#define SOFT_TXPIN 10

#define MAX_PAYLOAD_SIZE                    260

#define SERIAL_PORT                         Serial              // Serial class instance on which the BitPipe is found.

#define SERIAL_SPEED                        115200
#define SERIAL_CONFIG                       SERIAL_8N1

#define SIM_PIN                             ""                  // When SIM card is locked using a PIN
#define SIM_PUK                             ""                  // When too many bad PINs were tried, the PIN Unlock Key is necessary.

#define APN_CONFIG_HOST                     ""                // TODO: Enter you APN host address here
#define APN_CONFIG_USER                     ""
#define APN_CONFIG_PASS                     ""

// -------- MQTT: If you want a public MQTT server, check out cloudmqtt.com for trials.
#define MQTT_REMOTE_HOST                    ""                 // TODO: Enter the server host address here

#define MQTT_REMOTE_PORT                    1883               // TODO: Enter the server port here
#define MQTT_SECURE                         0
#define MQTT_CLEAN_SESSION                  1
#define MQTT_CLIENT_ID                      ""                  // TODO: Choose a unique client ID
#define MQTT_USER                           ""                  // TODO: Enter your instance username here
#define MQTT_PASS                           ""                  // TODO: Enter your instance password here

#define MQTT_SUB_TOP1                       "outputs/#"         // Topics to be used for to set the outputs

#define MQTT_PUBLISH_TOPIC_TEMPERATURE      "sensors/temperature"

// Local variables  ----------------------------------------------------------------------------------------------------

MQTTClient client((uint8_t*)MQTT_CLIENT_ID, &SERIAL_PORT, (uint8_t*)APN_CONFIG_HOST);

const uint8_t temperature=42;
unsigned long read_ts;
uint8_t i;
bool subscribed;

State old_state = NO_SIM_DETECTED;

SoftwareSerial RECALL_SERIAL(SOFT_RXPIN, SOFT_TXPIN);

// Local functions  ----------------------------------------------------------------------------------------------------

void setup()
{
    //For Blinker
    digitalWrite(LED_PIN, LOW);
    pinMode(LED_PIN, OUTPUT);

    subscribed = false;

    RECALL_SERIAL.begin(115200);

    client.setup(SERIAL_SPEED, SERIAL_CONFIG, &receive_data_action);
}

void loop()
{

    // Blinker ------------------------------------------------------------------------------------
    digitalWrite(LED_PIN, millis() % 1000 < 50 ? HIGH : LOW);

    // Connect ------------------------------------------------------------------------------------
    if(!client.isconnected())
    {
        client.connect((uint8_t*) MQTT_REMOTE_HOST, MQTT_REMOTE_PORT);
    }

    // Publish each 10 seconds --------------------------------------------------------------------
    if ((millis() - read_ts) > 10000UL)
    {
        client.publish((uint8_t*)MQTT_PUBLISH_TOPIC_TEMPERATURE, (uint8_t*) temperature, 0, 0);
        read_ts = millis();
    }

    // Subscribe -----------------------------------------------------------------------------------
    if (!subscribed && state == MQTT_CONNECTED && millis() < 50000UL)
    {
        client.subscribe((uint8_t*)MQTT_SUB_TOP1, 0);
        subscribed = true;
    }

    // MQTT main system loop  -----------------------------------------------------------------------
    client.loop();
}

void receive_data_action(uint8_t* topic, uint8_t topic_len,
                         uint8_t* data, uint8_t data_len,
                         uint8_t qos, uint8_t retained,
                         uint8_t complete)
{
	//Place your own code here to handle the data received from publications related to the subscribed topics

}

