/** ********************************************************************************************************************
 * @file        bp_api_devi.h
 * @author      Briowireless, Inc.
 * @date        2016-10-28
 * @brief       API layer for DEVICE INFORMATION requests to the BitPipe, and defining callback prototypes for replies
 *               and asynchronous posts sent from the BitPipe.
 *              See the BitPipe Serial API document for more information on requests below.
 * @version     API V1
 * @copyright   Briowireless, Inc., 2016, Licensed under BSD license. See LICENSE file for more details.
 *
 ** *******************************************************************************************************************/
#ifndef __BP_API_DEVI_H__
#define __BP_API_DEVI_H__


// Include directives  -------------------------------------------------------------------------------------------------

#include <platform_defs.h>
#include <message_protocol.h>

#ifdef __cplusplus
extern "C" {
#endif

// Preprocessor definitions --------------------------------------------------------------------------------------------

#define BP_API_DEVI_GET_AUTONOMOUS_MODE_CONFIG__ERR_NOT_CONFIGURED              0x0A    ///< Autonomous mode configuration is not set.

// Global Type definitions ---------------------------------------------------------------------------------------------

typedef void (*bp_api_devi_get_hw_version_reply)(uint8_t seq_nr, uint8_t status, uint8_t hw_ver_len, uint8_t* hw_ver);
typedef void (*bp_api_devi_get_fw_version_reply)(uint8_t seq_nr, uint8_t status, uint16_t boot_ver, uint8_t api_ver,
                                                 uint8_t bp_ver_len, uint8_t* bp_ver, uint8_t radio_ver_len, uint8_t* radio_ver);
typedef void (*bp_api_devi_get_product_name_reply)(uint8_t seq_nr, uint8_t status, uint8_t prod_name_len, uint8_t* prod_name);
typedef void (*bp_api_devi_get_vin_vio_voltage_reply)(uint8_t seq_nr, uint8_t status, uint16_t vin_voltage, uint16_t vio_voltage);
typedef void (*bp_api_devi_get_junction_temp_reply)(uint8_t seq_nr, uint8_t status, int8_t junction_temp);
typedef void (*bp_api_devi_set_low_power_reply)(uint8_t seq_nr, uint8_t status);
typedef void (*bp_api_devi_get_latest_fw_version_reply)(uint8_t seq_nr, uint8_t status, uint8_t fw_ver_len, uint8_t* fw_ver);
typedef void (*bp_api_devi_update_fw_reply)(uint8_t seq_nr, uint8_t status);
typedef void (*bp_api_devi_get_autonomous_mode_config_reply)(uint8_t seq_nr, uint8_t status, uint8_t secure, uint16_t ka_ival, uint16_t port,
                                                      uint8_t host_len, uint8_t* host, uint8_t user_len, uint8_t* user,
                                                      uint8_t pass_len, uint8_t* pass, uint8_t gpfx_len, uint8_t* gpfx);
typedef void (*bp_api_devi_set_autonomous_mode_config_reply)(uint8_t seq_nr, uint8_t status);


// Public Functions ----------------------------------------------------------------------------------------------------

uint8_t bp_api_devi_get_hw_version(mp_seq_nr* seq_nr);
uint8_t bp_api_devi_get_fw_version(mp_seq_nr* seq_nr);
uint8_t bp_api_devi_get_product_name(mp_seq_nr* seq_nr);
uint8_t bp_api_devi_get_vin_vio_voltage(mp_seq_nr* seq_nr);
uint8_t bp_api_devi_get_junction_temp(mp_seq_nr* seq_nr);
uint8_t bp_api_devi_set_low_power_mode(mp_seq_nr* seq_nr);
uint8_t bp_api_devi_get_latest_fw_version(mp_seq_nr* seq_nr);
uint8_t bp_api_devi_update_fw(mp_seq_nr* seq_nr, uint8_t* target, uint8_t target_len);
uint8_t bp_api_devi_get_autonomous_mode_config(mp_seq_nr* seq_nr);
uint8_t bp_api_devi_set_autonomous_mode_config(mp_seq_nr* seq_nr, uint8_t secure, uint16_t ka_ival, uint16_t port,
                                               uint8_t host_len, uint8_t* host, uint8_t user_len, uint8_t* user,
                                               uint8_t pass_len, uint8_t* pass, uint8_t gpfx_len, uint8_t* gpfx);

#ifdef __cplusplus
}
#endif

#endif // __BP_API_DEVI_H__
