/** ********************************************************************************************************************
 * @file        bp_api_mobi.h
 * @author      Briowireless, Inc.
 * @date        2016-10-28
 * @brief       API layer for MOBILE INFORMATION requests to the BitPipe, and defining callback prototypes for replies and
 *               asynchronous posts sent from the BitPipe.
 *              See the BitPipe Serial API document for more information on requests below.
 * @version     API V1
 * @copyright   Briowireless, Inc., 2016, Licensed under BSD license. See LICENSE file for more details.
 *
 ** *******************************************************************************************************************/
#ifndef __BP_API_MOBI_H__
#define __BP_API_MOBI_H__


// Include directives  -------------------------------------------------------------------------------------------------

#include <platform_defs.h>
#include <message_protocol.h>

#ifdef __cplusplus
extern "C" {
#endif

// Preprocessor definitions --------------------------------------------------------------------------------------------

#define BP_API_PWRON_RADIO__PWRON_METHOD_DCDC                   0x19    ///< Uses the DC-DC converter to power on the radio.
#define BP_API_PWRON_RADIO__PWRON_METHOD_BYPASS                 0x25    ///< Bypasses the DC-DC converter directly applying VIN to the wireless device.
#define BP_API_PWRON_RADIO__BACKOFF_ENABLED                     0x01    ///< Friendly Network Check is performed and may prevent radio power-on. RECOMMENDED
#define BP_API_PWRON_RADIO__BACKOFF_DISABLED                    0x00    ///< Friendly Network Check is NOT performed. Aggressive behavior is not prevented.
#define BP_API_PWRON_RADIO__STATUS_ALREADY_STARTED              0x0A    ///< The radio is aldready powered.
#define BP_API_PWRON_RADIO__STATUS_VIN_INCOMPAT_PWRON_METHOD    0x0B    ///< The voltage sensed at VIN doesn't allow this power-on method.
#define BP_API_PWRON_RADIO__STATUS_DENIED_FRIENDLY_NETWORK_CHK  0x0C    ///< Friendly Network Check prevented this power-on operation.

#define BP_API_PWROFF_RADIO__PWROFF_METHOD_GRACEFUL             0x00    ///< Disconects from mobile network and powers-off. RECOMMENDED
#define BP_API_PWROFF_RADIO__PWROFF_METHOD_HARD                 0x01    ///< Powers-off immediately. Note: this is not network friendly.

#define BP_API_RADIO_START_NOTIFY__REASON_PWRON_READY           0x00    ///< The radio started and the BitPipe is ready to handle radio-related operations.
#define BP_API_RADIO_START_NOTIFY__REASON_PWRON_TIMEOUT         0x01    ///< A timeout was detected while waiting for radio to boot up.
#define BP_API_RADIO_START_NOTIFY__REASON_UNEXPECTED_REBOOT     0x02    ///< An unexpected radio reboot occurred and the BitPipe is now ready.
#define BP_API_RADIO_START_NOTIFY__SIM_ACCESS_GRANTED           0x00    ///< SIM status is OK.
#define BP_API_RADIO_START_NOTIFY__SIM_DENIED_NEED_PIN          0x01    ///< SIM is locked by PIN. Need PIN to continue.
#define BP_API_RADIO_START_NOTIFY__SIM_DENIED_NEED_PUK          0x02    ///< SIM PIN is locked. Need PUK to continue.
#define BP_API_RADIO_START_NOTIFY__SIM_NOT_INSERTED             0x03    ///< There is no SIM card in the BitPipe.

#define BP_API_GET_RADIO_STATUS__RADIO_PWRON_NOT_READY          0x00    ///< The radio is powered-on but the BitPipe is not ready to handle radio-related operations.
#define BP_API_GET_RADIO_STATUS__RADIO_PWRON_READY              0x01    ///< The radio is powered-on and the BitPipe is ready to handle radio-related operations.
#define BP_API_GET_RADIO_STATUS__RADIO_PWROFF                   0x02    ///< The radio is powered-off.

#define BP_API_GET_MOBILE_CONN_STATUS__CONN_IND_NOT_CONNECTED   0x00    ///< Not connected to the mobile network.
#define BP_API_GET_MOBILE_CONN_STATUS__CONN_IND_CAMPING         0x01    ///< Camping on a mobile cell, but not authorized.
#define BP_API_GET_MOBILE_CONN_STATUS__CONN_IND_REGD_HOME       0x02    ///< Registered on home mobile network.
#define BP_API_GET_MOBILE_CONN_STATUS__CONN_IND_REGD_FOREIGN    0x03    ///< Registered on foreign mobile operator network (i.e.: roaming).


// Global Type definitions ---------------------------------------------------------------------------------------------

typedef void (*bp_api_mobi_power_on_radio_reply)(uint8_t seq_nr, uint8_t status);
typedef void (*bp_api_mobi_power_off_radio_reply)(uint8_t seq_nr, uint8_t status);
typedef void (*bp_api_mobi_radio_start_notify_post)(uint8_t seq_nr, uint8_t notify_reason, uint8_t sim_status);
typedef void (*bp_api_mobi_get_radio_status_reply)(uint8_t seq_nr, uint8_t status, uint8_t radio_status, uint8_t sim_status);
typedef void (*bp_api_mobi_get_imei_reply)(uint8_t seq_nr, uint8_t status, uint8_t* imei);
typedef void (*bp_api_mobi_get_iccid_reply)(uint8_t seq_nr, uint8_t status, uint8_t* iccid);
typedef void (*bp_api_mobi_get_imsi_reply)(uint8_t seq_nr, uint8_t status, uint8_t* imsi);
typedef void (*bp_api_mobi_get_own_phone_number_reply)(uint8_t seq_nr, uint8_t status, uint8_t msisdn_len, uint8_t* msisdn);
typedef void (*bp_api_mobi_get_apn_config_reply)(uint8_t seq_nr, uint8_t status, uint8_t host_len, uint8_t* host,
                                                 uint8_t user_len, uint8_t* user, uint8_t pass_len, uint8_t* pass);
typedef void (*bp_api_mobi_set_apn_config_reply)(uint8_t seq_nr, uint8_t status);
typedef void (*bp_api_mobi_set_sim_access_config_reply)(uint8_t seq_nr, uint8_t status);
typedef void (*bp_api_mobi_get_mobile_conn_status_reply)(uint8_t seq_nr, uint8_t status, uint8_t conn_ind, uint8_t link_tech,
                                                         uint16_t net_lac, uint32_t net_cell_id, uint8_t signal_strength,
                                                         uint8_t* net_mcc, uint8_t* net_mnc);
typedef void (*bp_api_mobi_mobile_conn_status_change_notify_post)(uint8_t seq_nr, uint8_t conn_ind, uint8_t link_tech,
                                                                  uint16_t net_lac, uint32_t net_cell_id);
typedef void (*bp_api_mobi_get_mobile_date_time_reply)(uint8_t seq_nr, uint8_t status, uint8_t year, uint8_t month, uint8_t day,
                                                       uint8_t hour, uint8_t minute, uint8_t second);


// Public Functions ----------------------------------------------------------------------------------------------------

uint8_t bp_api_mobi_power_on_radio(mp_seq_nr* seq_nr, uint8_t pwr_on_method, uint8_t back_off_enabled);
uint8_t bp_api_mobi_power_off_radio(mp_seq_nr* seq_nr, uint8_t pwr_off_method);
uint8_t bp_api_mobi_get_radio_status(mp_seq_nr* seq_nr);
uint8_t bp_api_mobi_get_imei(mp_seq_nr* seq_nr);
uint8_t bp_api_mobi_get_iccid(mp_seq_nr* seq_nr);
uint8_t bp_api_mobi_get_imsi(mp_seq_nr* seq_nr);
uint8_t bp_api_mobi_get_own_phone_number(mp_seq_nr* seq_nr);
uint8_t bp_api_mobi_get_apn_config(mp_seq_nr* seq_nr);
uint8_t bp_api_mobi_set_apn_config(mp_seq_nr* seq_nr, uint8_t host_len, uint8_t* host,
                                   uint8_t user_len, uint8_t* user, uint8_t pass_len, uint8_t* pass);
uint8_t bp_api_mobi_set_sim_access_config(mp_seq_nr* seq_nr, uint8_t pin_len, uint8_t* pin);
uint8_t bp_api_mobi_get_mobile_conn_status(mp_seq_nr* seq_nr);
uint8_t bp_api_mobi_get_mobile_date_time(mp_seq_nr* seq_nr);

#ifdef __cplusplus
}
#endif

#endif // __BP_API_MOBI_H__
