/** ********************************************************************************************************************
 * @file        bp_api_mqtt.h
 * @author      Briowireless, Inc.
 * @date        2016-11-08
 * @brief       API layer for MQTT requests and replies to the BitPipe, and defining callback prototypes for
 *               requests, replies and asynchronous posts sent from the BitPipe.
 *              See the BitPipe Serial API document for more information on requests below.
 * @version     API V1
 * @copyright   Briowireless, Inc., 2016, Licensed under BSD license. See LICENSE file for more details.
 *
 ** *******************************************************************************************************************/
#ifndef __BP_API_MQTT_H__
#define __BP_API_MQTT_H__


// Include directives  -------------------------------------------------------------------------------------------------

#include <platform_defs.h>
#include <message_protocol.h>

#ifdef __cplusplus
extern "C" {
#endif

// Preprocessor definitions --------------------------------------------------------------------------------------------

#define BP_API_MQTT_DISCONNECT__INVALID_CONNECTION_IDENTIFIER   0x0A    ///< Error: Invalid MQTT connection identifier

#define BP_API_MQTT_PUBLISH__CANNOT_PUBLISH_NOT_READY       	0x0B    ///< Cannot publish: Underlying TCP connection is not ready.

// Global Type definitions ---------------------------------------------------------------------------------------------

typedef void (*bp_api_mqtt_init_connection_reply)(uint8_t seq_nr, uint8_t status, uint8_t mqtt_conn_id);
typedef void (*bp_api_mqtt_connection_established_notify_post)(uint8_t seq_nr, uint8_t mqtt_conn_id);
typedef void (*bp_api_mqtt_subscribe_topics_reply)(uint8_t seq_nr, uint8_t status, uint8_t sub_status);
typedef void (*bp_api_mqtt_unsubscribe_topics_reply)(uint8_t seq_nr, uint8_t status);
typedef void (*bp_api_mqtt_publish_reply)(uint8_t seq_nr, uint8_t status);
typedef void (*bp_api_mqtt_publication_rx_notify_request)(uint8_t seq_nr, uint8_t mqtt_conn_id,
                                                          uint8_t qos, uint8_t retained, uint8_t complete,
                                                          uint8_t topic_len, uint8_t* topic,
                                                          uint8_t data_len, uint8_t* data);
typedef void (*bp_api_mqtt_disconnect_reply)(uint8_t seq_nr, uint8_t status);
typedef void (*bp_api_mqtt_disconnect_notify_post)(uint8_t seq_nr, uint8_t mqtt_conn_id, uint8_t reason);


// Public Functions ----------------------------------------------------------------------------------------------------

uint8_t bp_api_mqtt_init_connection(mp_seq_nr* seq_nr, uint8_t secure, uint8_t clean_session,
                                    uint8_t host_len, uint8_t* host, uint16_t port,
                                    uint8_t client_id_len, uint8_t* client_id,
                                    uint8_t user_len, uint8_t* user,
                                    uint8_t pass_len, uint8_t* pass,
                                    uint8_t ka_ival_5_secs);
uint8_t bp_api_mqtt_subscribe_topics(mp_seq_nr* seq_nr, uint8_t mqtt_conn_id, uint8_t qos1, uint8_t qos2, uint8_t qos3,
                                     uint8_t topic1_len, uint8_t* topic1,
                                     uint8_t topic2_len, uint8_t* topic2,
                                     uint8_t topic3_len, uint8_t* topic3);
uint8_t bp_api_mqtt_unsubscribe_topics(mp_seq_nr* seq_nr, uint8_t mqtt_conn_id,
                                       uint8_t topic1_len, uint8_t* topic1,
                                       uint8_t topic2_len, uint8_t* topic2,
                                       uint8_t topic3_len, uint8_t* topic3);
uint8_t bp_api_mqtt_publish(mp_seq_nr* seq_nr, uint8_t mqtt_conn_id, uint8_t qos, uint8_t retained, uint8_t send_indicator,
                            uint8_t topic_len, uint8_t* topic,
                            uint8_t data_len, uint8_t* data);
uint8_t bp_api_mqtt_publication_rx_notify_ack(mp_seq_nr ack_seq_nr);
uint8_t bp_api_mqtt_disconnect(mp_seq_nr* seq_nr, uint8_t mqtt_conn_id);

#ifdef __cplusplus
}
#endif

#endif // __BP_API_MQTT_H__
