/** ********************************************************************************************************************
 * @file        bp_api_sms.h
 * @author      Briowireless, Inc.
 * @date        2016-10-28
 * @brief       API layer for SMS requests and replies to the BitPipe, and defining callback prototypes for
 *               requests, replies and asynchronous posts sent from the BitPipe.
 *              See the BitPipe Serial API document for more information on requests below.
 * @version     API V1
 * @copyright   Briowireless, Inc., 2016, Licensed under BSD license. See LICENSE file for more details.
 *
 ** *******************************************************************************************************************/
#ifndef __BP_API_SMS_H__
#define __BP_API_SMS_H__


// Include directives  -------------------------------------------------------------------------------------------------

#include <platform_defs.h>
#include <message_protocol.h>

#ifdef __cplusplus
extern "C" {
#endif

// Preprocessor definitions --------------------------------------------------------------------------------------------

#define BP_API_SMS_SETUP_TRANSMISSION__CONCAT_SUPPORT_DISABLED                  0x00    ///< SMS concatenation support disabled.
#define BP_API_SMS_SETUP_TRANSMISSION__CONCAT_SUPPORT_ENABLED                   0x01    ///< SMS concatenation support enabled.
#define BP_API_SMS_SETUP_TRANSMISSION__ENCODING_7BITS_GSM0338                   0x00    ///< SMS encoding: 7-bits GSM03.38
#define BP_API_SMS_SETUP_TRANSMISSION__ENCODING_8BITS_BINARY                    0x01    ///< SMS encoding: 8-bits binary
#define BP_API_SMS_SETUP_TRANSMISSION__ENCODING_16BITS_UCS2                     0x02    ///< SMS encoding: 16-bits UCS2


// Global Type definitions ---------------------------------------------------------------------------------------------

typedef void (*bp_api_sms_setup_transmission_reply)(uint8_t seq_nr, uint8_t status);
typedef void (*bp_api_sms_message_append_send_reply)(uint8_t seq_nr, uint8_t status);
typedef void (*bp_api_sms_message_rx_notify_request)(uint8_t seq_nr, uint8_t encoding,
                                                     uint8_t year, uint8_t month, uint8_t day,
                                                     uint8_t hour, uint8_t minute, uint8_t second,
                                                     uint8_t tz_info_offset, uint8_t tz_info_sign,
                                                     uint8_t msisdn_len, uint8_t* msisdn);
typedef void (*bp_api_sms_message_part_notify_request)(uint8_t seq_nr, uint8_t last_part_indicator,
                                                       uint8_t data_len, uint8_t* data);


// Public Functions ----------------------------------------------------------------------------------------------------

uint8_t bp_api_sms_setup_transmission(mp_seq_nr* seq_nr, uint8_t concat_support, uint8_t encoding, uint8_t msisdn_len, uint8_t* msisdn);
uint8_t bp_api_sms_message_append_send(mp_seq_nr* seq_nr, uint8_t send_indicator, uint8_t data_len, uint8_t* data);
uint8_t bp_api_sms_message_rx_notify_ack(mp_seq_nr ack_seq_nr);
uint8_t bp_api_sms_message_part_notify_ack(mp_seq_nr ack_seq_nr);

#ifdef __cplusplus
}
#endif

#endif // __BP_API_SMS_H__
